import os
import shutil
import shlex
from pathlib import Path

from qtpy import QtCore, QtGui, QtWidgets
from qtpy.QtCore import Qt
import qtpy.compat as qtcompat


from .utils import translate as _, MONOSPACE_FONT, PIXMAPS, ICONS


class Config:
    def __init__(self):
        self.config_wdir = None
        self.script = None
        self.args = ""
        self.warmup = None
        self.config_env = ""
        self.config_stats = None
        self.config_kernprof = None

    @property
    def wdir(self):
        return self.config_wdir or self.default_wdir

    @property
    def default_wdir(self):
        return os.getcwd()

    @property
    def isvalid_wdir(self):
        return not self.config_wdir or Path(self.config_wdir).is_dir()

    @property
    def isvalid_script(self):
        if not self.script:
            return False
        # Script file must exist, either as an ablsolute path,
        # or relative to the working directory
        script_path = Path(self.config_wdir or "") / Path(self.script)
        return script_path.is_file()

    @property
    def isvalid_warmup(self):
        if not self.warmup:
            return True
        # Script file must exist, either as an ablsolute path,
        # or relative to the working directory
        warmup_path = Path(self.config_wdir or "") / Path(self.warmup)
        return warmup_path.is_file()

    @property
    def stats(self):
        return self.config_stats or self.default_stats

    @property
    def default_stats(self):
        return self.default_stats_for_script(self.script)

    def default_stats_for_script(self, script):
        if not script:
            return None
        else:
            return os.fspath(Path(script).with_suffix(".lprof"))

    @property
    def isvalid_stats(self):
        if not self.config_stats:
            return True
        return (
            not self.config_stats.endswith(("/", "\\"))
            and Path(self.config_stats).parent.is_dir()
        )

    @property
    def kernprof(self):
        return self.config_kernprof or self.default_kernprof

    @property
    def default_kernprof(self):
        return shutil.which("kernprof")

    @property
    def isvalid_kernprof(self):
        default_kernprof_path = not self.config_kernprof and self.default_kernprof
        return default_kernprof_path or Path(self.config_kernprof).is_file()

    @property
    def env(self):
        env = {}
        for var in shlex.split(self.config_env):
            name, value = var.split("=", maxsplit=1)
            env[name] = value
        return env

    @property
    def isvalid_env(self):
        try:
            self.env
        except ValueError:
            return False
        return True

    @property
    def isvalid(self):
        return bool(
            self.isvalid_wdir
            and self.isvalid_script
            and self.isvalid_warmup
            and self.isvalid_stats
            and self.isvalid_kernprof
            and self.isvalid_env
        )


class Ui_ConfigDialog(QtWidgets.QDialog):
    def __init__(self, parent, config):
        self.config = config

        QtWidgets.QDialog.__init__(self, parent)
        self.setup_ui()

        self.config_to_ui()

    def config_to_ui(self):
        self.wdirWidget.setText(self.config.config_wdir)
        self.scriptWidget.setText(self.config.script)
        self.argsWidget.setText(self.config.args)
        self.warmupWidget.setText(self.config.warmup)
        self.envWidget.setText(self.config.config_env)
        self.statsWidget.setText(self.config.config_stats)
        self.kernprofWidget.setText(self.config.config_kernprof)

        self.update()

    def ui_to_config(self, config=None):
        if config is None:
            config = self.config
        config.config_wdir = self.wdirWidget.text() or None
        config.script = self.scriptWidget.text() or None
        config.args = self.argsWidget.text()
        config.warmup = self.warmupWidget.text() or None
        config.config_env = self.envWidget.text()
        config.config_stats = self.statsWidget.text() or None
        config.config_kernprof = self.kernprofWidget.text() or None

    def update(self):
        self.wdirWidget.setPlaceholderText(self.config.default_wdir)
        self.update_stats_placeholder()
        self.kernprofWidget.setPlaceholderText(self.config.default_kernprof)
        self.on_wdirWidget_textChanged("")
        self.on_scriptWidget_textChanged("")
        self.on_warmupWidget_textChanged("")
        self.on_statsWidget_textChanged("")
        self.on_kernprofWidget_textChanged("")
        self.on_envWidget_textChanged("")

    def update_stats_placeholder(self):
        self.statsWidget.setPlaceholderText(
            self.config.default_stats_for_script(self.scriptWidget.text())
        )

    def setup_ui(self):
        # Dialog
        self.setObjectName("self")
        self.resize(600, 186)
        self.setModal(True)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.configLayout = QtWidgets.QFormLayout(self)
        self.configLayout.setObjectName("configLayout")
        row = 0

        # Working directory
        self.wdirLabel = QtWidgets.QLabel(self)
        self.configLayout.setWidget(
            row, QtWidgets.QFormLayout.LabelRole, self.wdirLabel
        )
        self.wdirLayout = QtWidgets.QHBoxLayout()
        self.wdirWidget = QtWidgets.QLineEdit(self)
        self.wdirWidget.setObjectName("wdirWidget")
        self.wdirLayout.addWidget(self.wdirWidget)
        self.wdirStatusLabel = QtWidgets.QLabel(self)
        self.wdirStatusLabel.setPixmap(PIXMAPS["NOK"])
        self.wdirLayout.addWidget(self.wdirStatusLabel)
        self.wdirButton = QtWidgets.QPushButton(self)
        self.wdirButton.setIcon(ICONS["DIRECTORY"])
        self.wdirButton.setObjectName("wdirButton")
        self.wdirLayout.addWidget(self.wdirButton)
        self.configLayout.setLayout(
            row, QtWidgets.QFormLayout.FieldRole, self.wdirLayout
        )
        self.wdirWidget.setValidator(ConfigValidator(self, "wdir"))
        row += 1

        # Python script
        self.scriptLabel = QtWidgets.QLabel(self)
        self.configLayout.setWidget(
            row, QtWidgets.QFormLayout.LabelRole, self.scriptLabel
        )
        self.scriptLayout = QtWidgets.QHBoxLayout()
        self.scriptWidget = QtWidgets.QLineEdit(self)
        self.scriptWidget.setObjectName("scriptWidget")
        self.scriptLayout.addWidget(self.scriptWidget)
        self.scriptStatusLabel = QtWidgets.QLabel(self)
        self.scriptStatusLabel.setPixmap(PIXMAPS["NOK"])
        self.scriptLayout.addWidget(self.scriptStatusLabel)
        self.scriptButton = QtWidgets.QPushButton(self)
        self.scriptButton.setIcon(ICONS["READFILE"])
        self.scriptButton.setObjectName("scriptButton")
        self.scriptLayout.addWidget(self.scriptButton)
        self.configLayout.setLayout(
            row, QtWidgets.QFormLayout.FieldRole, self.scriptLayout
        )
        self.scriptWidget.setValidator(ConfigValidator(self, "script"))
        row += 1

        # Scripts args
        self.argsLabel = QtWidgets.QLabel(self)
        self.configLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.argsLabel)
        self.argsWidget = QtWidgets.QLineEdit(self)
        self.argsWidget.setFont(MONOSPACE_FONT)
        self.argsWidget.setObjectName("argsWidget")
        self.configLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.argsWidget)
        row += 1

        # Python script
        self.warmupLabel = QtWidgets.QLabel(self)
        self.configLayout.setWidget(
            row, QtWidgets.QFormLayout.LabelRole, self.warmupLabel
        )
        self.warmupLayout = QtWidgets.QHBoxLayout()
        self.warmupWidget = QtWidgets.QLineEdit(self)
        self.warmupWidget.setObjectName("warmupWidget")
        self.warmupLayout.addWidget(self.warmupWidget)
        self.warmupStatusLabel = QtWidgets.QLabel(self)
        self.warmupStatusLabel.setPixmap(PIXMAPS["NOK"])
        self.warmupLayout.addWidget(self.warmupStatusLabel)
        self.warmupButton = QtWidgets.QPushButton(self)
        self.warmupButton.setIcon(ICONS["READFILE"])
        self.warmupButton.setObjectName("warmupButton")
        self.warmupLayout.addWidget(self.warmupButton)
        self.configLayout.setLayout(
            row, QtWidgets.QFormLayout.FieldRole, self.warmupLayout
        )
        self.warmupWidget.setValidator(ConfigValidator(self, "warmup"))
        row += 1

        # Environment variables
        self.envLabel = QtWidgets.QLabel(self)
        self.configLayout.setWidget(row, QtWidgets.QFormLayout.LabelRole, self.envLabel)
        self.envLayout = QtWidgets.QHBoxLayout()
        self.envWidget = QtWidgets.QLineEdit(self)
        self.envWidget.setObjectName("envWidget")
        self.envLayout.addWidget(self.envWidget)
        self.envStatusLabel = QtWidgets.QLabel(self)
        self.envStatusLabel.setPixmap(PIXMAPS["NOK"])
        self.envLayout.addWidget(self.envStatusLabel)
        self.configLayout.setLayout(
            row, QtWidgets.QFormLayout.FieldRole, self.envLayout
        )
        self.envWidget.setValidator(ConfigValidator(self, "env"))
        row += 1

        # Stats filename
        self.statsLabel = QtWidgets.QLabel(self)
        self.configLayout.setWidget(
            row, QtWidgets.QFormLayout.LabelRole, self.statsLabel
        )
        self.statsLayout = QtWidgets.QHBoxLayout()
        self.statsWidget = QtWidgets.QLineEdit(self)
        self.statsWidget.setObjectName("statsWidget")
        self.statsLayout.addWidget(self.statsWidget)
        self.statsStatusLabel = QtWidgets.QLabel(self)
        self.statsStatusLabel.setPixmap(PIXMAPS["NOK"])
        self.statsLayout.addWidget(self.statsStatusLabel)
        self.statsButton = QtWidgets.QPushButton(self)
        self.statsButton.setIcon(ICONS["BLANKFILE"])
        self.statsButton.setObjectName("statsButton")
        self.statsLayout.addWidget(self.statsButton)
        self.configLayout.setLayout(
            row, QtWidgets.QFormLayout.FieldRole, self.statsLayout
        )
        self.statsWidget.setValidator(ConfigValidator(self, "stats"))
        row += 1

        # kernprof executable
        self.kernprofLabel = QtWidgets.QLabel(self)
        self.configLayout.setWidget(
            row, QtWidgets.QFormLayout.LabelRole, self.kernprofLabel
        )
        self.kernprofLayout = QtWidgets.QHBoxLayout()
        self.kernprofWidget = QtWidgets.QLineEdit(self)
        self.kernprofWidget.setObjectName("kernprofWidget")
        self.kernprofLayout.addWidget(self.kernprofWidget)
        self.kernprofStatusLabel = QtWidgets.QLabel(self)
        self.kernprofStatusLabel.setPixmap(PIXMAPS["NOK"])
        self.kernprofLayout.addWidget(self.kernprofStatusLabel)
        self.kernprofButton = QtWidgets.QPushButton(self)
        self.kernprofButton.setIcon(ICONS["READFILE"])
        self.kernprofButton.setObjectName("kernprofButton")
        self.kernprofLayout.addWidget(self.kernprofButton)
        self.configLayout.setLayout(
            row, QtWidgets.QFormLayout.FieldRole, self.kernprofLayout
        )
        self.kernprofWidget.setValidator(ConfigValidator(self, "kernprof"))
        row += 1

        # Buttons
        self.buttonBox = QtWidgets.QDialogButtonBox(self)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.profileButton = self.buttonBox.addButton(
            _("Profile"), QtWidgets.QDialogButtonBox.AcceptRole
        )
        self.profileButton.setObjectName("profileButton")
        self.buttonBox.setStandardButtons(
            QtWidgets.QDialogButtonBox.Save | QtWidgets.QDialogButtonBox.Cancel
        )
        self.buttonBox.setObjectName("buttonBox")
        self.configLayout.setWidget(
            row, QtWidgets.QFormLayout.SpanningRole, self.buttonBox
        )
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        row += 1

        # Finalization
        self.retranslate_ui()
        QtCore.QMetaObject.connectSlotsByName(self)

    def retranslate_ui(self):
        self.setWindowTitle(_("Line Profiler GUI - Profiling configuration"))
        self.wdirLabel.setText(_("Working directory"))
        self.wdirButton.setText(_("Select..."))
        self.scriptLabel.setText(_("Python script"))
        self.scriptButton.setText(_("Select..."))
        self.warmupLabel.setText(_("Warm up script"))
        self.warmupButton.setText(_("Select..."))
        self.argsLabel.setText(_("Script args"))
        self.envLabel.setText(_("Environment variables"))
        self.statsLabel.setText(_("Stats filename"))
        self.statsButton.setText(_("Select..."))
        self.kernprofLabel.setText(_("<tt>kernprof</tt> path"))
        self.kernprofButton.setText(_("Select..."))

    @QtCore.Slot()
    def accept(self):
        self.ui_to_config()
        QtWidgets.QDialog.accept(self)
        super().accept()

    @QtCore.Slot()
    def on_profileButton_clicked(self):
        self.accept()
        self.parent().profile()

    def update_profileButton_enabled(self):
        config = Config()
        self.ui_to_config(config)
        self.profileButton.setEnabled(config.isvalid)

    @QtCore.Slot()
    def on_wdirButton_clicked(self):
        filename = qtcompat.getexistingdirectory(
            self, _("Select Python script"), self.wdirWidget.text(),
        )
        if filename:
            self.wdirWidget.setText(filename)

    def display_status(self, widget, indicator):
        indicator.setVisible(not widget.hasAcceptableInput())
        self.update_profileButton_enabled()

    @QtCore.Slot(str)
    def on_wdirWidget_textChanged(self, text):
        self.display_status(self.wdirWidget, self.wdirStatusLabel)

    @QtCore.Slot(str)
    def on_scriptWidget_textChanged(self, text):
        self.display_status(self.scriptWidget, self.scriptStatusLabel)
        self.update_stats_placeholder()

    @QtCore.Slot(str)
    def on_warmupWidget_textChanged(self, text):
        self.display_status(self.warmupWidget, self.warmupStatusLabel)

    @QtCore.Slot(str)
    def on_statsWidget_textChanged(self, text):
        self.display_status(self.statsWidget, self.statsStatusLabel)

    @QtCore.Slot(str)
    def on_kernprofWidget_textChanged(self, text):
        self.display_status(self.kernprofWidget, self.kernprofStatusLabel)

    @QtCore.Slot(str)
    def on_envWidget_textChanged(self, text):
        self.display_status(self.envWidget, self.envStatusLabel)

    @QtCore.Slot()
    def on_scriptButton_clicked(self):
        filename, _selfilter = qtcompat.getopenfilename(
            self,
            _("Select Python script"),
            self.scriptWidget.text(),
            _("Python scripts") + " (*.py ; *.pyw)",
        )
        if filename:
            self.scriptWidget.setText(filename)

    @QtCore.Slot()
    def on_warmupButton_clicked(self):
        filename, _selfilter = qtcompat.getopenfilename(
            self,
            _("Select Python warmup script"),
            self.warmupWidget.text(),
            _("Python scripts") + " (*.py ; *.pyw)",
        )
        if filename:
            self.warmupWidget.setText(filename)

    @QtCore.Slot()
    def on_statsButton_clicked(self):
        filename, _selfilter = qtcompat.getsavefilename(
            self,
            _("Select stats filename"),
            self.statsWidget.text(),
            _("Stats filename") + " (* ; *.lprof)",
        )
        if filename:
            self.statsWidget.setText(filename)

    @QtCore.Slot()
    def on_kernprofButton_clicked(self):
        filename, _selfilter = qtcompat.getopenfilename(
            self,
            _("Select kernprof executable"),
            self.kernprofWidget.text() or self.config.default_kernprof,
            _("kernprof executable") + " (kernprof*)",
        )
        if filename:
            self.kernprofWidget.setText(filename)


class ConfigValidator(QtGui.QValidator):
    def __init__(self, configDialog, widgetID):
        QtGui.QValidator.__init__(self)
        self.configDialog = configDialog
        self.widgetID = widgetID

    @QtCore.Slot(str, int)
    def validate(self, text, pos):
        config = Config()
        self.configDialog.ui_to_config(config)
        return (
            QtGui.QValidator.Acceptable
            if getattr(config, f"isvalid_{self.widgetID}")
            else QtGui.QValidator.Intermediate,
            text,
            pos,
        )



import linecache
import pickle
import inspect
import hashlib
import os
import functools
import math
import subprocess

from qtpy import QtCore, QtGui, QtWidgets
from qtpy.QtCore import Qt

from .utils import translate as _, MONOSPACE_FONT


def load_profile_data(filename):
    """Load line profiler data saved by kernprof module"""
    # stats has the following layout :
    # stats.timings =
    #     {(filename1, line_no1, function_name1):
    #         [(line_no1, hits1, total_time1),
    #          (line_no2, hits2, total_time2)],
    #      (filename2, line_no2, function_name2):
    #         [(line_no1, hits1, total_time1),
    #          (line_no2, hits2, total_time2),
    #          (line_no3, hits3, total_time3)]}
    # stats.unit = time_factor
    with open(filename, "rb") as fid:
        stats = pickle.load(fid)

    data = []
    for func_info, func_stats in stats.timings.items():
        # func_info is a tuple containing (filename, line, function name)
        func_data = FunctionData(func_info, func_stats, stats.unit)
        data.append(func_data)
    return data


class FunctionData:
    def __init__(self, func_info, stats, time_unit):
        self.filename, self.start_line_no, self.name = func_info
        self.filename = os.path.normpath(self.filename)
        self.total_time = 0.0
        self.was_called = False
        self.time_unit = time_unit

        self.load_code()
        self.parse_stats(stats)

    @property
    def id(self):
        return (self.filename, self.name)

    def load_code(self):
        # Note : linecache cache was checked at start of profiling
        # This way if the file has changed since the code ran there
        # is a chance that the correct version was in cache and we get
        # the correct lines.
        all_lines = linecache.getlines(self.filename)
        self.code_lines = inspect.getblock(all_lines[self.start_line_no :])

    def parse_stats(self, stats):
        self.line_data = []
        self.total_time = 0.0
        self.was_called = False
        next_stat_index = 0
        for line_no, code_line in enumerate(self.code_lines):
            line_no += self.start_line_no + 1  # Lines start at 1
            code_line = code_line.rstrip()

            # stats contains data for runned lines only : (line_no, hits, total_time)
            if next_stat_index >= len(stats) or line_no != stats[next_stat_index][0]:
                # Line didn't run
                hits, line_total_time = None, None
            else:
                # Compute line times
                hits, line_total_time = stats[next_stat_index][1:]
                line_total_time *= self.time_unit
                self.total_time += line_total_time
                next_stat_index += 1
                self.was_called = True
            self.line_data.append(
                LineData(self, line_no, code_line, line_total_time, hits)
            )

    @functools.cached_property
    def color(self):
        """Choose deteministic unique color for the function"""
        md5 = hashlib.md5((self.filename + self.name).encode("utf8")).hexdigest()
        hue = (int(md5[:2], 16) - 68) % 360  # avoid blue (unreadable)
        color = QtGui.QColor.fromHsv(hue, 200, 255)
        return color

    def __iter__(self):
        yield from self.line_data


class LineData:
    __slots__ = "_func_data line_no code total_time hits filename".split()

    def __init__(self, func_data, line_no, code, total_time, hits):
        self._func_data = func_data
        self.line_no = line_no
        self.code = code
        self.total_time = total_time
        self.hits = hits
        self.filename = func_data.filename

    @property
    def percent_str(self):
        if self.total_time is None:
            return ""
        percent = 100 * self.total_time / self._func_data.total_time
        return f"{percent:.1f}"

    @property
    def time_str(self):
        if self.total_time is None:
            return ""
        return f"{self.total_time * 1e3:.3f}"

    @property
    def per_hit_str(self):
        if self.total_time is None:
            return ""
        return f"{self.total_time / self.hits * 1e3:.3f}"

    @property
    def hits_str(self):
        if self.hits is None:
            return ""
        return str(self.hits)

    @property
    def color(self):
        color = QtGui.QColor(self._func_data.color)  # Makes a copy
        ratio = self.total_time / self._func_data.total_time
        ratio = math.log10(9 * ratio + 1)  # Logarithmic while keeping 0 <= ratio <= 1
        color.setAlphaF(ratio)
        return QtGui.QBrush(color)


class ResultsTreeWidget(QtWidgets.QTreeWidget):
    """Tree widget to view line_profiler results"""

    column_header_text = [
        _("Line #"),
        _("Hits"),
        _("Time (ms)"),
        _("Per Hit (ms)"),
        _("% Time"),
        _("Line Contents"),
    ]
    COL_0 = 0
    COL_NO = 0
    COL_HITS = 1
    COL_TIME = 2
    COL_PERHIT = 3
    COL_PERCENT = 4
    COL_LINE = 5
    COL_FILE_LINE = 0  # Not displayed but used to store data as Qt.UserRole

    CODE_NOT_RUN_COLOR = QtGui.QBrush(QtGui.QColor.fromRgb(128, 128, 128, 200))

    def __init__(self, parent):
        QtWidgets.QTreeWidget.__init__(self, parent)
        self.setup_ui()

        self.profiledata = None

        self.lock_expanded_tracking = False
        self.expanded_functions = set()

    def setup_ui(self):
        self.setColumnCount(len(self.column_header_text))
        self.setHeaderLabels(self.column_header_text)
        self.header().setDefaultAlignment(Qt.AlignCenter)
        self.setProperty("showDropIndicator", False)
        self.setUniformRowHeights(True)
        self.setSortingEnabled(False)
        self.setItemsExpandable(True)
        self.setDragEnabled(False)

        self.itemActivated.connect(self.item_activated)
        self.itemCollapsed.connect(self.item_collapsed)
        self.itemExpanded.connect(self.item_expanded)

        self.updateColonsVisible()

    def show_tree(self, profiledata):
        """Populate the tree with line profiler data and display it."""
        # Remember scrollbar position
        scrollbar = self.verticalScrollBar()
        scroll = scrollbar.value()

        # Fill the widget with the profile data
        self.populate_tree(profiledata)

        # Adjust column width to fit all content
        self.lock_expanded_tracking = True
        self.expandAll()
        self.lock_expanded_tracking = False
        for col in range(self.columnCount() - 1):
            self.resizeColumnToContents(col)

        # Restore expanded state for each function
        root = self.invisibleRootItem()
        if self.topLevelItemCount() > 1:
            self.lock_expanded_tracking = True
            for index in range(root.childCount()):
                item = root.child(index)
                func_id = item.data(self.COL_FILE_LINE, Qt.UserRole)
                item.setExpanded(func_id in self.expanded_functions)
            self.lock_expanded_tracking = False
        else:
            # Since we forced the function to be expanded, store it for consistency
            item = root.child(0)
            func_id = item.data(self.COL_FILE_LINE, Qt.UserRole)
            self.expanded_functions.add(func_id)

        # Restore scrollbar position
        scrollbar.setValue(scroll)

    def populate_tree(self, profiledata):
        """Create each item (and associated data) in the tree"""
        # Clear before re-populating
        self.clear()

        # Display a warning in case of empty profile data
        if not profiledata:
            self.warn_no_data()
            return

        for func_data in profiledata:
            # Function name and position
            func_item = QtWidgets.QTreeWidgetItem(self)
            func_item.setData(
                self.COL_0,
                Qt.DisplayRole,
                _(
                    '{func_name} ({time_ms:.3f}ms) in file "{filename}", line {line_no}'
                ).format(
                    filename=func_data.filename,
                    line_no=func_data.start_line_no,
                    func_name=func_data.name,
                    time_ms=func_data.total_time * 1e3,
                ),
            )
            func_item.setData(self.COL_0, Qt.UserRole, func_data.id)
            func_item.setFirstColumnSpanned(True)
            if not func_data.was_called:
                func_item.setForeground(self.COL_0, self.CODE_NOT_RUN_COLOR)

            # Lines of code
            for line_data in func_data:
                line_item = QtWidgets.QTreeWidgetItem(func_item)
                self.fill_line_item(line_item, line_data)

                # Color background
                if line_data.total_time is not None:
                    color = line_data.color
                    for col in range(self.columnCount()):
                        line_item.setBackground(col, color)
                else:
                    for col in range(self.columnCount()):
                        line_item.setForeground(col, self.CODE_NOT_RUN_COLOR)

    def fill_line_item(self, item, line_data):
        item.setData(
            self.COL_FILE_LINE, Qt.UserRole, (line_data.filename, line_data.line_no),
        )
        item.setData(self.COL_NO, Qt.DisplayRole, line_data.line_no)
        item.setData(
            self.COL_PERCENT, Qt.DisplayRole, line_data.percent_str,
        )
        item.setTextAlignment(self.COL_PERCENT, Qt.AlignCenter)
        item.setData(self.COL_TIME, Qt.DisplayRole, line_data.time_str)
        item.setTextAlignment(self.COL_TIME, Qt.AlignCenter)
        item.setData(self.COL_PERHIT, Qt.DisplayRole, line_data.per_hit_str)
        item.setTextAlignment(self.COL_PERHIT, Qt.AlignCenter)
        item.setData(self.COL_HITS, Qt.DisplayRole, line_data.hits_str)
        item.setTextAlignment(self.COL_HITS, Qt.AlignCenter)
        item.setData(self.COL_LINE, Qt.DisplayRole, line_data.code)
        item.setFont(self.COL_LINE, MONOSPACE_FONT)

    def warn_no_data(self):
        warn_item = QtWidgets.QTreeWidgetItem(self)
        warn_item.setData(
            self.COL_0,
            Qt.DisplayRole,
            _("No timings to display. Did you forget to add @profile decorators ?"),
        )
        warn_item.setFirstColumnSpanned(True)
        warn_item.setTextAlignment(self.COL_0, Qt.AlignCenter)
        font = warn_item.font(self.COL_0)
        font.setStyle(QtGui.QFont.StyleItalic)
        warn_item.setFont(self.COL_0, font)

    def updateColonsVisible(self):
        settings = QtCore.QSettings()
        for col in range(5):
            self.setColumnHidden(
                col, not settings.value(f"column{col+1}Visible", True, bool)
            )

    @QtCore.Slot(QtWidgets.QTreeWidgetItem)
    def item_activated(self, item):
        # Skip parent lines
        if item.isFirstColumnSpanned():
            return

        # Retrieve command
        settings = QtCore.QSettings()
        editor_command = settings.value("editorCommand", "").strip()
        if not editor_command:
            return

        # Replace special values
        filename, line_no = item.data(self.COL_FILE_LINE, Qt.UserRole)
        filename = filename.replace("\\", "\\\\")
        # fmt: off
        editor_command = (
            editor_command
            .replace("{file}", f'"{filename}"')
            .replace("{line}", str(line_no))
        )
        # fmt: on

        # Run
        try:
            subprocess.Popen(editor_command, shell=False)
        except FileNotFoundError:
            subprocess.Popen(editor_command, shell=True)

    @QtCore.Slot(QtWidgets.QTreeWidgetItem)
    def item_collapsed(self, item):
        # Skip child lines
        if self.lock_expanded_tracking or not item.isFirstColumnSpanned():
            return
        func_id = item.data(self.COL_FILE_LINE, Qt.UserRole)
        self.expanded_functions.discard(func_id)

    @QtCore.Slot(QtWidgets.QTreeWidgetItem)
    def item_expanded(self, item):
        # Skip child lines
        if self.lock_expanded_tracking or not item.isFirstColumnSpanned():
            return
        func_id = item.data(self.COL_FILE_LINE, Qt.UserRole)
        self.expanded_functions.add(func_id)



import datetime
import os

from qtpy import QtCore, QtGui, QtWidgets
from qtpy.QtCore import Qt
import qtpy.compat as qtcompat

from .config import Config, Ui_ConfigDialog
from .tree import ResultsTreeWidget, load_profile_data
from .settings import UI_SettingsDialog
from .utils import translate as _, MONOSPACE_FONT, ICONS, PIXMAPS
from .process import KernprofRun

LINE_PROFILER_DOC_URL = "https://github.com/pyutils/line_profiler#id2"


class UI_MainWindow(QtWidgets.QMainWindow):
    def __init__(self):
        self.config = Config()

        QtWidgets.QMainWindow.__init__(self)
        self.setup_ui()
        self.kernprof_run = KernprofRun(self.config)
        self.connect()

        self.profile_start_time = None

    def setup_ui(self):
        settings = QtCore.QSettings()
        # Main window
        # app.setWindowIcon(QIcon(_WINDOW_ICON))
        self.resize(800, 600)

        # Results widget
        self.resultsTreeWidget = ResultsTreeWidget(self)
        self.resultsTreeWidget.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.resultsTreeWidget.setMinimumSize(300, 50)
        self.setCentralWidget(self.resultsTreeWidget)

        # Output widget
        self.dockOutputWidget = DockOutputWidget(self)
        self.dockOutputWidget.setObjectName("dockOutputWidget")
        self.addDockWidget(Qt.BottomDockWidgetArea, self.dockOutputWidget)

        # Actions
        self.actionCollapse_all = QtWidgets.QAction(self)
        self.actionCollapse_all.setIcon(ICONS["COLLAPSE"])
        self.actionExpand_all = QtWidgets.QAction(self)
        self.actionExpand_all.setIcon(ICONS["EXPAND"])
        self.actionRun = QtWidgets.QAction(self)
        self.actionRun.setIcon(ICONS["START"])
        self.actionAbort = QtWidgets.QAction(self)
        self.actionAbort.setIcon(ICONS["STOP"])
        self.actionShowOutput = self.dockOutputWidget.toggleViewAction()
        self.actionShowOutput.setIcon(ICONS["INFO"])
        self.actionLoadLprof = QtWidgets.QAction(self)
        self.actionLoadLprof.setIcon(ICONS["READFILE"])
        self.actionQuit = QtWidgets.QAction(self)
        self.actionQuit.setIcon(ICONS["ABORT"])
        self.actionConfigure = QtWidgets.QAction(self)
        self.actionConfigure.setIcon(ICONS["CONFIG"])
        self.actionSettings = QtWidgets.QAction(self)
        self.actionSettings.setIcon(ICONS["SETTINGS"])
        self.actionLine_profiler_documentation = QtWidgets.QAction(self)
        self.actionLine_profiler_documentation.setIcon(ICONS["HELP"])
        self.actionAbout_Qt = QtWidgets.QAction(self)
        self.actionAbout_Qt.setIcon(ICONS["QT"])

        # Menu bar
        self.menubar = QtWidgets.QMenuBar(self)
        self.setMenuBar(self.menubar)

        # Profiling menu
        self.menuProfiling = QtWidgets.QMenu(self.menubar)
        self.menuProfiling.addAction(self.actionConfigure)
        self.menuProfiling.addSeparator()
        self.menuProfiling.addAction(self.actionRun)
        self.menuProfiling.addAction(self.actionAbort)
        self.menuProfiling.addAction(self.actionShowOutput)
        self.menuProfiling.addSeparator()
        self.menuProfiling.addAction(self.actionLoadLprof)
        self.menuProfiling.addSeparator()
        self.menuProfiling.addAction(self.actionQuit)
        self.menubar.addAction(self.menuProfiling.menuAction())

        # Display Menu
        self.menuDisplay = QtWidgets.QMenu(self.menubar)
        self.menuDisplay.addAction(self.actionCollapse_all)
        self.menuDisplay.addAction(self.actionExpand_all)
        self.menuDisplay.addSeparator()
        self.menuDisplay.addAction(self.actionSettings)
        self.menubar.addAction(self.menuDisplay.menuAction())

        # Help Menu
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.addAction(self.actionLine_profiler_documentation)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionAbout_Qt)
        self.menubar.addAction(self.menuHelp.menuAction())

        # Toolbar
        self.toolBar = QtWidgets.QToolBar(self)
        self.toolBar.setObjectName("toolBar")
        self.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.toolBar.addAction(self.actionConfigure)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionRun)
        self.toolBar.addAction(self.actionAbort)
        self.toolBar.addAction(self.actionShowOutput)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionCollapse_all)
        self.toolBar.addAction(self.actionExpand_all)
        self.toolBar.addSeparator()
        self.historyCombo = QtWidgets.QComboBox(self)
        self.historyCombo.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.toolBar.addWidget(self.historyCombo)

        # Statusbar
        self.statusbar = QtWidgets.QStatusBar(self)
        self.setStatusBar(self.statusbar)
        self.statusbar_running_indicator = QtWidgets.QLabel()
        self.statusbar_running_indicator.setPixmap(PIXMAPS["RUNNING"])
        self.statusbar_running_indicator.hide()
        self.statusbar_running_indicator_timer = QtCore.QTimer(self)
        self.statusbar_running_indicator_timer.timeout.connect(
            lambda: self.statusbar_running_indicator.setVisible(
                not self.statusbar_running_indicator.isVisible()
            )
        )
        self.statusbar.addPermanentWidget(self.statusbar_running_indicator)
        self.statusbar_time = QtWidgets.QLabel()
        self.statusbar.addWidget(self.statusbar_time)

        self.settingsDialog = UI_SettingsDialog(self)

        # Finalization
        self.retranslate_ui()
        self.read_settings()
        self.set_running_state(False)

    def connect(self):
        self.actionCollapse_all.triggered.connect(self.resultsTreeWidget.collapseAll)
        self.actionExpand_all.triggered.connect(self.resultsTreeWidget.expandAll)
        self.actionConfigure.triggered.connect(self.configure)
        self.actionSettings.triggered.connect(self.settingsDialog.show)
        self.actionRun.triggered.connect(self.profile)
        self.actionAbort.triggered.connect(self.kernprof_run.kill)
        self.actionShowOutput.toggled.connect(self.dockOutputWidget.setVisible)
        self.actionLoadLprof.triggered.connect(self.selectLprof)
        self.actionQuit.triggered.connect(QtWidgets.QApplication.instance().quit)
        self.actionLine_profiler_documentation.triggered.connect(
            lambda: QtGui.QDesktopServices.openUrl(QtCore.QUrl(LINE_PROFILER_DOC_URL))
        )
        self.actionAbout_Qt.triggered.connect(QtWidgets.QApplication.aboutQt)
        self.kernprof_run.output_text.connect(self.dockOutputWidget.append_log_text)
        self.kernprof_run.output_error.connect(self.dockOutputWidget.append_log_error)
        self.settingsDialog.accepted.connect(self.resultsTreeWidget.updateColonsVisible)
        self.historyCombo.currentIndexChanged.connect(self.load_history)

    def retranslate_ui(self):
        self.update_window_title()
        self.toolBar.setWindowTitle(_("Tool bar"))
        self.menuDisplay.setTitle(_("&Display"))
        self.menuProfiling.setTitle(_("&Profiling"))
        self.menuHelp.setTitle(_("&Help"))
        self.actionCollapse_all.setText(_("&Collapse all"))
        self.actionExpand_all.setText(_("&Expand all"))
        self.actionRun.setText(_("&Profile"))
        self.actionRun.setShortcut(_("F5"))
        self.actionAbort.setText(_("&Stop"))
        self.actionAbort.setShortcut(_("F6"))
        self.actionShowOutput.setText(_("&Console output"))
        self.actionShowOutput.setShortcut(_("F7"))
        self.actionLoadLprof.setText(_("&Load data..."))
        self.actionLoadLprof.setShortcut(_("Ctrl+O"))
        self.actionQuit.setText(_("&Quit"))
        self.actionQuit.setShortcut(_("Ctrl+Q"))
        self.actionConfigure.setText(_("&Configuration..."))
        self.actionConfigure.setShortcut(_("F4"))
        self.actionSettings.setText(_("&Settings..."))
        self.actionLine_profiler_documentation.setText(
            _("&Line-profiler documentation...")
        )
        self.actionLine_profiler_documentation.setShortcut(_("F1"))
        self.actionAbout_Qt.setText(_("&About Qt..."))

    def update_window_title(self):
        title = _("Line Profiler GUI")
        if self.config.script:
            title += f" - {self.config.script}"
        self.setWindowTitle(title)

    def closeEvent(self, event):
        self.write_settings()
        QtWidgets.QMainWindow.closeEvent(self, event)

    def write_settings(self):
        settings = QtCore.QSettings()
        settings.beginGroup("MainWindow")
        settings.setValue("geometry", self.saveGeometry())
        settings.setValue("state", self.saveState())
        settings.endGroup()

    def read_settings(self):
        settings = QtCore.QSettings()
        settings.beginGroup("MainWindow")
        self.restoreGeometry(settings.value("geometry"))
        self.restoreState(settings.value("state"))
        settings.endGroup()

    @QtCore.Slot()
    def selectLprof(self):
        filename, _selfilter = qtcompat.getopenfilename(
            self,
            _("Select line profiler data"),
            "",
            _("Line profiler data") + " (*.lprof);; " + _("All files") + " (*.*)",
        )
        if filename:
            self.load_lprof(filename)

    @QtCore.Slot()
    def configure(self):
        Ui_ConfigDialog(self, self.config).exec()
        self.update_window_title()

    @QtCore.Slot()
    def profile(self):
        # Configuration dialog in case of invalid config
        if not self.config.isvalid:
            self.configure()
            if not self.config.isvalid:
                return

        # Start process
        self.dockOutputWidget.clear()
        process = self.kernprof_run.prepare()
        process.stateChanged.connect(self.set_running_state)
        process.finished.connect(self.process_finished)
        self.profile_start_time = datetime.datetime.now()
        self.kernprof_run.start()

    @QtCore.Slot(QtCore.QProcess.ProcessState)
    def set_running_state(self, running):
        self.actionRun.setEnabled(not running)
        self.actionAbort.setEnabled(running)
        self.actionConfigure.setEnabled(not running)
        self.statusbar_running_indicator.setVisible(running)
        self.dockOutputWidget.set_running_state(running)
        if running:
            self.setCursor(Qt.WaitCursor)
            self.statusbar_running_indicator_timer.start(800)
        else:
            self.unsetCursor()
            self.statusbar_running_indicator_timer.stop()
            # status icons are set in self.process_finished()

    @QtCore.Slot(int, QtCore.QProcess.ExitStatus)
    def process_finished(self, exit_code, exit_status):
        profile_stop_time = datetime.datetime.now()
        profile_duration = profile_stop_time - self.profile_start_time
        profile_time_str = profile_stop_time.strftime("%X")
        profile_duration_str = str(profile_duration).lstrip("0:")
        if profile_duration_str.startswith("."):
            profile_duration_str = "0" + profile_duration_str
        self.statusbar_time.setText(
            _("Last profiling ended at {time} and ran for {duration}s").format(
                time=profile_time_str, duration=profile_duration_str,
            )
        )
        self.dockOutputWidget.set_exit_state(exit_code or exit_status)

        title = _("{duration}s at {time}").format(
            duration=profile_duration_str, time=profile_time_str
        )
        self.load_lprof(self.config.stats, title)

    def load_lprof(self, lprof_file, title=None):
        profile_data = load_profile_data(lprof_file)
        if not title:
            time = datetime.datetime.now().strftime("%X")
            name = os.path.basename(lprof_file)
            title = _("{name} at {time}").format(name=name, time=time)
        self.historyCombo.insertItem(0, title, profile_data)
        self.historyCombo.setCurrentIndex(0)

    @QtCore.Slot(int)
    def load_history(self, index):
        if index < 0:
            return
        self.resultsTreeWidget.show_tree(self.historyCombo.currentData())


class DockOutputWidget(QtWidgets.QDockWidget):
    def __init__(self, parent):
        QtWidgets.QDockWidget.__init__(self, parent)
        self.setup_ui()

    def setup_ui(self):
        # Console ouput widget
        self.outputWidget = QtWidgets.QPlainTextEdit()
        self.outputWidget.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.outputWidget.setReadOnly(True)
        self.outputWidget.setTextInteractionFlags(
            QtCore.Qt.LinksAccessibleByKeyboard
            | QtCore.Qt.LinksAccessibleByMouse
            | QtCore.Qt.TextBrowserInteraction
            | QtCore.Qt.TextSelectableByKeyboard
            | QtCore.Qt.TextSelectableByMouse
        )
        self.outputWidget.setFont(MONOSPACE_FONT)
        self.outputWidget.setMinimumSize(300, 50)
        self.setWidget(self.outputWidget)

    def set_running_state(self, running):
        actionShowOutput = self.toggleViewAction()
        actionShowOutput.setIcon(ICONS["RUNNING" if running else "INFO"])
        if running:
            self.setWindowTitle(_("{} Console output").format("🔄"))
            # self.setWindowTitle() overrides the action text
            # We reset it without the icon to avoid a ugly menu entry
            actionShowOutput.setText(_("&Console output"))
        else:
            self.setWindowTitle(_("Console output"))

    def set_exit_state(self, error):
        actionShowOutput = self.toggleViewAction()
        if error:
            self.show()
            self.activateWindow()
            self.outputWidget.setFocus(Qt.OtherFocusReason)
            actionShowOutput.setIcon(ICONS["WARNING"])
            self.setWindowTitle(_("{} Console output").format("⚠"))
        else:
            actionShowOutput.setIcon(ICONS["INFO"])
            self.setWindowTitle(_("{} Console output").format("ⓘ"))
        # self.setWindowTitle() overrides the action text
        # We reset it without the icon to avoid a ugly menu entry
        actionShowOutput.setText(_("&Console output"))

    @QtCore.Slot(str)
    def append_log_text(self, text):
        self.outputWidget.appendPlainText(text)

    @QtCore.Slot(str)
    def append_log_error(self, text):
        self.outputWidget.appendHtml(f'<p style="color:red;white-space:pre">{text}</p>')

    def clear(self):
        self.outputWidget.clear()



import sys
import argparse

from qtpy import QtCore, QtWidgets

from .gui import UI_MainWindow
from .utils import _icons_factory
from . import __version__


def positive_float(value):
    val = float(value)
    if val <= 0:
        raise argparse.ArgumentError
    return val


def commandline_args():
    """Manage arguments with argparse"""

    parser = argparse.ArgumentParser(
        description="Run, profile a python script and display results."
    )
    parser.add_argument(
        "-V", "--version", action="version", version=f"%(prog)s v{__version__}"
    )
    parser.add_argument(
        "-l", "--lprof", help="Display data from a .lprof file",
    )
    parser.add_argument(
        "-r", "--run", action="store_true", help="Profile the python script on launch"
    )
    parser.add_argument(
        "-o", "--outfile", help="Save stats to OUTFILE (default: 'scriptname.lprof')",
    )
    parser.add_argument(
        "-s", "--setup", help="Python script to execute before the code to profile"
    )
    parser.add_argument("script", nargs="?", help="The python script file to run")
    parser.add_argument("args", nargs="...", help="Optional script arguments")

    options = parser.parse_args()

    options.args = " ".join(options.args)

    return options


def main():
    options = commandline_args()

    # Create Qt application
    app = QtWidgets.QApplication([])
    _icons_factory()

    # Used for QSettings
    QtCore.QCoreApplication.setOrganizationName("OpenPyUtils")
    QtCore.QCoreApplication.setApplicationName("Line Profiler Gui")

    # Create main window
    win = UI_MainWindow()
    win.show()

    if options.lprof:
        win.load_lprof(options.lprof)

    win.config.script = options.script
    win.config.args = options.args
    win.config.warmup = options.setup
    win.config.outfile = options.outfile
    if options.script:
        win.update_window_title()
        if options.run:
            win.profile()
        else:
            win.configure()
    elif not options.lprof:
        win.configure()

    # Run Qt event loop
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



from functools import partial

from qtpy import QtCore, QtGui, QtWidgets

translate = partial(QtCore.QCoreApplication.translate, "self")


MONOSPACE_FONT = QtGui.QFont("Monospace")
MONOSPACE_FONT.setStyleHint(QtGui.QFont.Monospace)
MONOSPACE_FONT.setPointSize(9)

_ICON_IDS = {
    "OK": QtWidgets.QStyle.SP_DialogApplyButton,
    "NOK": QtWidgets.QStyle.SP_DialogCancelButton,  # Icon invalid field
    "START": QtWidgets.QStyle.SP_MediaPlay,  # actionRun
    "STOP": QtWidgets.QStyle.SP_MediaStop,  # actionAbort
    "ABORT": QtWidgets.QStyle.SP_BrowserStop,  # actionQuit
    "CONFIG": QtWidgets.QStyle.SP_FileDialogDetailedView,  # actionConfigure
    "SETTINGS": QtWidgets.QStyle.SP_FileDialogListView,  # actionSettings
    "BLANKFILE": QtWidgets.QStyle.SP_FileIcon,  # statsButton
    "READFILE": QtWidgets.QStyle.SP_FileDialogContentsView,  # scriptButton, kernprofButton
    "DIRECTORY": QtWidgets.QStyle.SP_DirIcon,  # wdirButton
    "EXPAND": QtWidgets.QStyle.SP_ToolBarVerticalExtensionButton,  # actionExpand_all
    "COLLAPSE": QtWidgets.QStyle.SP_ToolBarHorizontalExtensionButton,  # actionCollapse_all
    "QT": QtWidgets.QStyle.SP_TitleBarMenuButton,  # actionAbout_Qt
    "HELP": QtWidgets.QStyle.SP_DialogHelpButton,  # actionLine_profiler_documentation
    "INFO": QtWidgets.QStyle.SP_MessageBoxInformation,  # actionShowOutput
    "WARNING": QtWidgets.QStyle.SP_MessageBoxWarning,  # actionShowOutput
    "ERROR": QtWidgets.QStyle.SP_MessageBoxCritical,
    "RUNNING": QtWidgets.QStyle.SP_BrowserReload,  # statusbar_running_indicator, actionShowOutput
}

ICONS = {}
PIXMAPS = {}
PIXMAP_SIZE = 16


def _icons_factory():
    for k, v in _ICON_IDS.items():
        icon = QtWidgets.QApplication.style().standardIcon(v)
        ICONS[k] = icon
        PIXMAPS[k] = icon.pixmap(PIXMAP_SIZE, PIXMAP_SIZE)



from qtpy import QtCore, QtWidgets

from .utils import translate as _

EDITOR_COMMAND_EXAMPLES = [
    "",
    "code --goto {file}:{line}",
    '"C:\\\\Program Files (x86)\\\\Notepad++\\\\Notepad++" {file} -n{line}',
    "vim +{line} {file}",
]


class UI_SettingsDialog(QtWidgets.QDialog):
    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self, parent)
        self.setup_ui()

    def setup_ui(self):
        self.resize(600, 200)
        self.setModal(True)
        self.mainLayout = QtWidgets.QVBoxLayout(self)

        # Editor command
        self.editorCommandGroupBox = QtWidgets.QGroupBox(self)
        self.editorCommandLayout = QtWidgets.QVBoxLayout(self.editorCommandGroupBox)
        self.editorCommandLayout.setObjectName("editorCommandLayout")
        self.editorCommandLabel = QtWidgets.QLabel(self.editorCommandGroupBox)
        self.editorCommandLabel.setWordWrap(True)
        self.editorCommandLayout.addWidget(self.editorCommandLabel)
        self.editorCommandCombo = QtWidgets.QComboBox(self.editorCommandGroupBox)
        self.editorCommandCombo.setEditable(True)
        for cmd in EDITOR_COMMAND_EXAMPLES:
            self.editorCommandCombo.addItem(cmd)
        self.editorCommandLayout.addWidget(self.editorCommandCombo)
        self.mainLayout.addWidget(self.editorCommandGroupBox)

        # Columns visibility
        self.columnsGroupBox = QtWidgets.QGroupBox(self)
        self.columnsLayout = QtWidgets.QHBoxLayout(self.columnsGroupBox)
        self.checkBox_1 = QtWidgets.QCheckBox(self.columnsGroupBox)
        self.checkBox_1.setChecked(True)
        self.columnsLayout.addWidget(self.checkBox_1)
        self.checkBox_2 = QtWidgets.QCheckBox(self.columnsGroupBox)
        self.checkBox_2.setChecked(True)
        self.columnsLayout.addWidget(self.checkBox_2)
        self.checkBox_3 = QtWidgets.QCheckBox(self.columnsGroupBox)
        self.checkBox_3.setChecked(True)
        self.columnsLayout.addWidget(self.checkBox_3)
        self.checkBox_4 = QtWidgets.QCheckBox(self.columnsGroupBox)
        self.checkBox_4.setChecked(True)
        self.columnsLayout.addWidget(self.checkBox_4)
        self.checkBox_5 = QtWidgets.QCheckBox(self.columnsGroupBox)
        self.checkBox_5.setChecked(True)
        self.columnsLayout.addWidget(self.checkBox_5)
        self.mainLayout.addWidget(self.columnsGroupBox)

        # Button box
        self.buttonBox = QtWidgets.QDialogButtonBox(self)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(
            QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok
        )
        self.mainLayout.addWidget(self.buttonBox)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

        # Spacer
        spacerItem = QtWidgets.QSpacerItem(
            0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding
        )
        self.mainLayout.addItem(spacerItem)

        self.retranslate_ui()
        self.reject()  # Load settings values

    def retranslate_ui(self):
        _translate = QtCore.QCoreApplication.translate
        self.setWindowTitle(_("Line Profiler GUI - Settings"))
        self.editorCommandGroupBox.setTitle(_("Open file"))
        self.editorCommandLabel.setText(
            _(
                "Command to run when a line is double-clicked in the profiling result panel. <tt>{file}</tt> will be replaced by the stringified filename, and <tt>{line}</tt> will be replaced by the line number."
            )
        )
        self.columnsGroupBox.setTitle(_("Visible colums"))
        self.checkBox_1.setText(_("Line #"))
        self.checkBox_2.setText(_("Hits"))
        self.checkBox_3.setText(_("Time (ms)"))
        self.checkBox_4.setText(_("Per Hit (ms)"))
        self.checkBox_5.setText(_("% Time"))

    @QtCore.Slot()
    def accept(self):
        settings = QtCore.QSettings()
        settings.setValue("editorCommand", self.editorCommandCombo.currentText())
        settings.setValue("column1Visible", self.checkBox_1.isChecked())
        settings.setValue("column2Visible", self.checkBox_2.isChecked())
        settings.setValue("column3Visible", self.checkBox_3.isChecked())
        settings.setValue("column4Visible", self.checkBox_4.isChecked())
        settings.setValue("column5Visible", self.checkBox_5.isChecked())
        QtWidgets.QDialog.accept(self)

        self.parent()

    @QtCore.Slot()
    def reject(self):
        settings = QtCore.QSettings()
        self.editorCommandCombo.setCurrentText(settings.value("editorCommand", ""))
        self.checkBox_1.setChecked(settings.value("column1Visible", True, bool))
        self.checkBox_2.setChecked(settings.value("column2Visible", True, bool))
        self.checkBox_3.setChecked(settings.value("column3Visible", True, bool))
        self.checkBox_4.setChecked(settings.value("column4Visible", True, bool))
        self.checkBox_5.setChecked(settings.value("column5Visible", True, bool))
        QtWidgets.QDialog.reject(self)



import os
import shlex
import linecache

from qtpy import QtCore

from .utils import translate as _

LOCALE_CODEC = QtCore.QTextCodec.codecForLocale()


class KernprofRun(QtCore.QObject):
    output_text = QtCore.Signal(str)
    output_error = QtCore.Signal(str)

    def __init__(self, config):
        QtCore.QObject.__init__(self)
        self.config = config

        self.process = None
        self.p_args = None

    def prepare(self):
        self.process = QtCore.QProcess(self)
        self.process.readyReadStandardOutput.connect(self.read_output)
        self.process.readyReadStandardError.connect(self.read_error)

        # Manage environment
        qenv = QtCore.QProcessEnvironment.systemEnvironment()
        for name, value in self.config.env.items():
            qenv.insert(name, value)
        self.process.setProcessEnvironment(qenv)

        self.process.setWorkingDirectory(self.config.wdir)
        filename = self.config.script
        warmup = self.config.warmup
        if os.name == "nt":
            # On Windows, one has to replace backslashes by slashes to avoid
            # confusion with escape characters (otherwise, for example, '\t'
            # will be interpreted as a tabulation):
            filename = os.path.normpath(filename).replace(os.sep, "/")
            if warmup:
                warmup = os.path.normpath(warmup).replace(os.sep, "/")
        self.p_args = ["-l", "-o", self.config.stats]
        if warmup:
            self.p_args.extend(["--setup", warmup])
        self.p_args.append(filename)
        if self.config.args:
            self.p_args.extend(shlex.split(self.config.args))

        return self.process

    def start(self):
        self.process.start(
            self.config.kernprof,
            self.p_args,
            QtCore.QIODevice.ReadOnly | QtCore.QIODevice.Unbuffered,
        )

        # Update file cache, to be able to load the code from profiled files in the state that they were run (as much as possible...)
        linecache.checkcache()

        running = self.process.waitForStarted()
        if not running:
            self.output_error.emit(_("ERROR: Process failed to start"))

    @QtCore.Slot()
    def read_output(self):
        qbytearray = self.process.readAllStandardOutput()
        self.output_text.emit(LOCALE_CODEC.toUnicode(qbytearray))

    @QtCore.Slot()
    def read_error(self):
        qbytearray = self.process.readAllStandardError()
        self.output_error.emit(LOCALE_CODEC.toUnicode(qbytearray))

    @QtCore.Slot()
    def kill(self):
        if self.process is not None and self.process.state() == QtCore.QProcess.Running:
            self.process.kill()
            self.process.waitForFinished()
